@@ -9,7 +9,8 @@ import matplotlib as mpl
 from fedbase.utils.utils import get_targets
 from fedbase.utils import femnist
 import os
-import pickle
+# import pickle
+import json
 import datetime as d
 import math
 import pandas as pd
@@ -72,7 +73,7 @@ class data_process:
         # # print labels
         # print(' '.join('%5s' % classes[labels[j]] for j in range(batch_size)))
 
-    def split_dataset(self, num_nodes, alpha, method='dirichlet', train_dataset = None, test_dataset = None):
+    def split_dataset(self, num_nodes, alpha, method='dirichlet', train_dataset = None, test_dataset = None, plot_show = False):
         train_dataset = self.train_dataset if train_dataset is None else train_dataset
         test_dataset = self.test_dataset if test_dataset is None else test_dataset
         train_targets, test_targets = get_targets(train_dataset), get_targets(test_dataset)
@@ -95,49 +96,53 @@ class data_process:
                     len(labels), 1).repeat(1, num_nodes)
                 l_test = test_label_size.reshape(
                     len(labels), 1).repeat(1, num_nodes)
-                # print(l_test)
-                if method == 'dirichlet':
-                    p = torch.tensor(np.round(np.random.dirichlet(np.repeat(alpha, num_nodes), len(labels)), round(math.log(len(test_dataset)/len(labels),10))))
-                    # print(torch.sum(p,axis=1))
-                elif method == 'class':
-                    p = np.zeros((len(labels), 1))
-                    J = np.random.choice(len(labels), alpha, replace=False)
-                    p[J] = 1
-                    for k in range(1, num_nodes):
-                        x = np.zeros((len(labels), 1))
-                        J = np.random.choice(len(labels), alpha, replace=False)
-                        x[J] = 1
-                        p = np.concatenate((p, x), axis=1)
-                    p = p / np.repeat((p.sum(axis=1)+10**-10).reshape(len(labels), 1), num_nodes, axis=1)
-                # print(p.sum(axis=1),p)
-                train_size = torch.round(l_train*p).int()
-                test_size = torch.round(l_test*p).int()
+                
                 train_splited = []
                 test_splited = []
-                train_label_index = []
-                test_label_index = []
-                for j in range(len(labels)):
-                    train_label_index.append([(train_targets== labels[j]).nonzero(as_tuple=True)[
-                                             0][offset-length:offset] for offset, length in zip(_accumulate(train_size[j, :]), train_size[j, :])])
-                    test_label_index.append([(test_targets== labels[j]).nonzero(as_tuple=True)[
-                                            0][offset-length:offset] for offset, length in zip(_accumulate(test_size[j, :]), test_size[j, :])])
-                # how to deal with 0?
-                for i in range(num_nodes):
-                    if len(ConcatDataset([Subset(test_dataset, test_label_index[j][i]) for j in range(len(labels))]))>0:
-                        train_splited.append(ConcatDataset(
-                            [Subset(train_dataset, train_label_index[j][i]) for j in range(len(labels))]))
-                        test_splited.append(ConcatDataset(
-                            [Subset(test_dataset, test_label_index[j][i]) for j in range(len(labels))]))
-                if len(test_splited)<num_nodes:
-                    # print(self.dataset_name, len(train_splited),len(test_splited))
+                while len(test_splited) <= num_nodes//2:
+                    # print(l_test)
+                    if method == 'dirichlet':
+                        p = torch.tensor(np.round(np.random.dirichlet(np.repeat(alpha, num_nodes), len(labels)), round(math.log(len(test_dataset)/len(labels),10))))
+                        # print(torch.sum(p,axis=1))
+                    elif method == 'class':
+                        p = np.zeros((len(labels), 1))
+                        J = np.random.choice(len(labels), alpha, replace=False)
+                        p[J] = 1
+                        for k in range(1, num_nodes):
+                            x = np.zeros((len(labels), 1))
+                            J = np.random.choice(len(labels), alpha, replace=False)
+                            x[J] = 1
+                            p = np.concatenate((p, x), axis=1)
+                        p = p / np.repeat((p.sum(axis=1)+10**-10).reshape(len(labels), 1), num_nodes, axis=1)
+                    # print(p.sum(axis=1),p)
+                    train_size = torch.round(l_train*p).int()
+                    test_size = torch.round(l_test*p).int()
+                    train_label_index = []
+                    test_label_index = []
+                    for j in range(len(labels)):
+                        train_label_index.append([(train_targets== labels[j]).nonzero(as_tuple=True)[
+                                                0][offset-length:offset] for offset, length in zip(_accumulate(train_size[j, :]), train_size[j, :])])
+                        test_label_index.append([(test_targets== labels[j]).nonzero(as_tuple=True)[
+                                                0][offset-length:offset] for offset, length in zip(_accumulate(test_size[j, :]), test_size[j, :])])
+                    # how to deal with 0?
+                    for i in range(num_nodes):
+                        if len(ConcatDataset([Subset(test_dataset, test_label_index[j][i]) for j in range(len(labels))]))>5: # 0-10, to control the minimun length
+                            train_splited.append(ConcatDataset(
+                                [Subset(train_dataset, train_label_index[j][i]) for j in range(len(labels))]))
+                            test_splited.append(ConcatDataset(
+                                [Subset(test_dataset, test_label_index[j][i]) for j in range(len(labels))]))
+                while len(test_splited)<num_nodes:
+                    # print(self.dataset_name,len(test_splited),num_nodes-len(test_splited))
                     random_index = np.random.choice(range(len(test_splited)), num_nodes-len(test_splited), replace=True)
                     train_splited = train_splited + [train_splited[i] for i in range(len(train_splited)) if i in random_index]           
                     test_splited = test_splited + [test_splited[i] for i in range(len(test_splited)) if i in random_index]  
-
-                self.plot_split(labels, train_splited)    
+                if plot_show:
+                    self.plot_split(labels, train_splited)
+                # print(min([len(i) for i in train_splited]))
+                # print(min([len(i) for i in test_splited]))
                 return train_splited, test_splited, self.dataset_name +'_'+ str(num_nodes)+'_'+ str(alpha)+'_'+ str(method)
         
-    def split_dataset_groupwise(self, num0, alpha0, method0, num1, alpha1, method1, train_dataset = None, test_dataset = None):
+    def split_dataset_groupwise(self, num0, alpha0, method0, num1, alpha1, method1, train_dataset = None, test_dataset = None, plot_show = False):
         train_dataset = self.train_dataset if train_dataset is None else train_dataset
         test_dataset = self.test_dataset if test_dataset is None else test_dataset
         train_targets = get_targets(train_dataset)
@@ -150,11 +155,12 @@ class data_process:
             test_splited += test_tmp
         #plot
         labels = torch.unique(train_targets)
-        self.plot_split(labels, train_splited)
+        if plot_show:
+            self.plot_split(labels, train_splited)
         return train_splited, test_splited, self.dataset_name +'_'+ str(num0)+'_'+ str(alpha0)+'_'+ str(method0)+'_'+ str(num1)+'_'+ str(alpha1)+'_'+ str(method1)
 
     def plot_split(self, labels, train_splited):
-        return None
+        # return None
         train_size = []
         for x in train_splited:
             tmp = []
@@ -171,30 +177,32 @@ class data_process:
                 train_size[:i], 0), label=str(int(labels[i])))
         # plt.title("Data distribution of dataset")
         plt.legend()
-        mpl.rcParams['figure.dpi'] = 300
+        # plt.title('Group-wise Non-IID Setting', fontsize = 20)
+        plt.xlabel('Dataset size', fontsize=16)
+        plt.ylabel('Client ID', fontsize=16)
         plt.show()
 
 def log(file_name, nodes, server):
-    local_file = './log/' + file_name + "_" + d.datetime.now().strftime("%m%d_%H%M%S")+'_'+str(np.random.choice(10**3)) + ".pkl"
+    local_file = './log/' + file_name + "_" + d.datetime.now().strftime("%m%d_%H%M%S")+'_'+str(np.random.choice(10**3)) + ".json"
     log = {}
     log['node'] = {}
     for i in range(len(nodes)):
-        log['node'][str(i)] = nodes[i].accuracy
+        log['node'][str(i)] = nodes[i].test_metrics
     try:
-        log['server'] = server.accuracy
+        log['server'] = server.test_metrics
         log['clustering'] = server.clustering
     except:
         print('No server')
     # pd.to_pickle(log, local_file)
     Path(local_file).parent.mkdir(parents=True, exist_ok=True)
-    with  open(local_file, 'wb') as handle:
-        pickle.dump(log, handle, protocol=pickle.HIGHEST_PROTOCOL)
+    with  open(local_file, 'w') as handle:
+        json.dump(log, handle, indent=4)
 
     # read
     if os.path.exists(local_file):
-        with open(local_file, 'rb') as f:
-            log = pickle.load(f)
-            print(log)
+        with open(local_file, 'r') as f:
+            log = json.load(f)
+            # print(log)
 
 # dt = data_process('mnist')
 # # dt.split_dataset(50, 2, method='class')
